from insights.parsers.ceilometer_log import CeilometerCentralLog
from insights.parsers.ceilometer_log import CeilometerCollectorLog
from insights.parsers.ceilometer_log import CeilometerComputeLog
from insights.tests import context_wrap

from datetime import datetime

CENTRAL_LOG = """
2016-11-09 14:38:08.484 31654 WARNING oslo_reports.guru_meditation_report [-] Guru meditation now registers SIGUSR1 and SIGUSR2 by default for backward compatibility. SIGUSR1 will no longer be registered in a future release, so please use SIGUSR2 to generate reports.
2016-11-09 14:38:09.711 31723 INFO ceilometer.declarative [-] Definitions: {'metric': [{'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.10.1.3.1', 'type': 'lambda x: float(str(x))', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.cpu.load.1min', 'unit': 'process'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.10.1.3.2', 'type': 'lambda x: float(str(x))', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.cpu.load.5min', 'unit': 'process'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.10.1.3.3', 'type': 'lambda x: float(str(x))', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.cpu.load.15min', 'unit': 'process'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.11.9.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.cpu.util', 'unit': '%'}, {'snmp_inspector': {'post_op': '_post_op_disk', 'oid': '1.3.6.1.4.1.2021.9.1.6', 'type': 'int', 'matching_type': 'type_prefix', 'metadata': {'device': {'oid': '1.3.6.1.4.1.2021.9.1.3', 'type': 'str'}, 'path': {'oid': '1.3.6.1.4.1.2021.9.1.2', 'type': 'str'}}}, 'type': 'gauge', 'name': 'hardware.disk.size.total', 'unit': 'KB'}, {'snmp_inspector': {'post_op': '_post_op_disk', 'oid': '1.3.6.1.4.1.2021.9.1.8', 'type': 'int', 'matching_type': 'type_prefix', 'metadata': {'device': {'oid': '1.3.6.1.4.1.2021.9.1.3', 'type': 'str'}, 'path': {'oid': '1.3.6.1.4.1.2021.9.1.2', 'type': 'str'}}}, 'type': 'gauge', 'name': 'hardware.disk.size.used', 'unit': 'KB'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.4.5.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.memory.total', 'unit': 'KB'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.4.6.0', 'type': 'int', 'matching_type': 'type_exact', 'post_op': '_post_op_memory_avail_to_used'}, 'type': 'gauge', 'name': 'hardware.memory.used', 'unit': 'KB'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.4.3.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.memory.swap.total', 'unit': 'KB'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.4.4.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.memory.swap.avail', 'unit': 'KB'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.4.14.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.memory.buffer', 'unit': 'KB'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.4.15.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.memory.cached', 'unit': 'KB'}, {'snmp_inspector': {'post_op': '_post_op_net', 'oid': '1.3.6.1.2.1.2.2.1.10', 'type': 'int', 'matching_type': 'type_prefix', 'metadata': {'mac': {'oid': '1.3.6.1.2.1.2.2.1.6', 'type': "lambda x: x.prettyPrint().replace('0x', '')"}, 'speed': {'oid': '1.3.6.1.2.1.2.2.1.5', 'type': 'lambda x: int(x) / 8'}, 'name': {'oid': '1.3.6.1.2.1.2.2.1.2', 'type': 'str'}}}, 'type': 'cumulative', 'name': 'hardware.network.incoming.bytes', 'unit': 'B'}, {'snmp_inspector': {'post_op': '_post_op_net', 'oid': '1.3.6.1.2.1.2.2.1.16', 'type': 'int', 'matching_type': 'type_prefix', 'metadata': {'mac': {'oid': '1.3.6.1.2.1.2.2.1.6', 'type': "lambda x: x.prettyPrint().replace('0x', '')"}, 'speed': {'oid': '1.3.6.1.2.1.2.2.1.5', 'type': 'lambda x: int(x) / 8'}, 'name': {'oid': '1.3.6.1.2.1.2.2.1.2', 'type': 'str'}}}, 'type': 'cumulative', 'name': 'hardware.network.outgoing.bytes', 'unit': 'B'}, {'snmp_inspector': {'post_op': '_post_op_net', 'oid': '1.3.6.1.2.1.2.2.1.20', 'type': 'int', 'matching_type': 'type_prefix', 'metadata': {'mac': {'oid': '1.3.6.1.2.1.2.2.1.6', 'type': "lambda x: x.prettyPrint().replace('0x', '')"}, 'speed': {'oid': '1.3.6.1.2.1.2.2.1.5', 'type': 'lambda x: int(x) / 8'}, 'name': {'oid': '1.3.6.1.2.1.2.2.1.2', 'type': 'str'}}}, 'type': 'cumulative', 'name': 'hardware.network.outgoing.errors', 'unit': 'packet'}, {'snmp_inspector': {'oid': '1.3.6.1.2.1.4.10.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'cumulative', 'name': 'hardware.network.ip.outgoing.datagrams', 'unit': 'datagrams'}, {'snmp_inspector': {'oid': '1.3.6.1.2.1.4.3.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'cumulative', 'name': 'hardware.network.ip.incoming.datagrams', 'unit': 'datagrams'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.11.11.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.system_stats.cpu.idle', 'unit': '%'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.11.57.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'cumulative', 'name': 'hardware.system_stats.io.outgoing.blocks', 'unit': 'blocks'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.11.58.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'cumulative', 'name': 'hardware.system_stats.io.incoming.blocks', 'unit': 'blocks'}]}
2016-11-09 14:38:09.986 31723 WARNING oslo_config.cfg [-] Option "rpc_backend" from group "DEFAULT" is deprecated for removal.  Its value may be silently ignored in the future.
2016-11-09 14:38:10.040 31723 INFO ceilometer.pipeline [-] Config file: {'sources': [{'interval': 600, 'meters': ['*'], 'name': 'meter_source', 'sinks': ['meter_sink']}, {'interval': 600, 'meters': ['cpu'], 'name': 'cpu_source', 'sinks': ['cpu_sink', 'cpu_delta_sink']}, {'interval': 600, 'meters': ['disk.read.bytes', 'disk.read.requests', 'disk.write.bytes', 'disk.write.requests', 'disk.device.read.bytes', 'disk.device.read.requests', 'disk.device.write.bytes', 'disk.device.write.requests'], 'name': 'disk_source', 'sinks': ['disk_sink']}, {'interval': 600, 'meters': ['network.incoming.bytes', 'network.incoming.packets', 'network.outgoing.bytes', 'network.outgoing.packets'], 'name': 'network_source', 'sinks': ['network_sink']}], 'sinks': [{'publishers': ['notifier://'], 'transformers': None, 'name': 'meter_sink'}, {'publishers': ['notifier://'], 'transformers': [{'name': 'rate_of_change', 'parameters': {'target': {'scale': '100.0 / (10**9 * (resource_metadata.cpu_number or 1))', 'type': 'gauge', 'name': 'cpu_util', 'unit': '%'}}}], 'name': 'cpu_sink'}, {'publishers': ['notifier://'], 'transformers': [{'name': 'delta', 'parameters': {'target': {'name': 'cpu.delta'}, 'growth_only': True}}], 'name': 'cpu_delta_sink'}, {'publishers': ['notifier://'], 'transformers': [{'name': 'rate_of_change', 'parameters': {'source': {'map_from': {'name': '(disk\\.device|disk)\\.(read|write)\\.(bytes|requests)', 'unit': '(B|request)'}}, 'target': {'map_to': {'name': '\\1.\\2.\\3.rate', 'unit': '\\1/s'}, 'type': 'gauge'}}}], 'name': 'disk_sink'}, {'publishers': ['notifier://'], 'transformers': [{'name': 'rate_of_change', 'parameters': {'source': {'map_from': {'name': 'network\\.(incoming|outgoing)\\.(bytes|packets)', 'unit': '(B|packet)'}}, 'target': {'map_to': {'name': 'network.\\1.\\2.rate', 'unit': '\\1/s'}, 'type': 'gauge'}}}], 'name': 'network_sink'}]}
2016-11-09 14:38:10.041 31723 INFO ceilometer.pipeline [-] detected decoupled pipeline config format
"""


COLLECTOR_LOG = """
2016-11-09 14:32:41.099 4259 WARNING oslo_config.cfg [-] Option "max_retries" from group "database" is deprecated. Use option "max_retries" from group "storage".
2016-11-09 14:36:35.464 4204 INFO cotyledon [-] Caught SIGTERM signal, graceful exiting of master process
2016-11-09 14:36:35.465 4259 INFO cotyledon [-] Caught signal (15) during service initialisation, delaying it
2016-11-09 14:38:07.280 31638 WARNING oslo_reports.guru_meditation_report [-] Guru meditation now registers SIGUSR1 and SIGUSR2 by default for backward compatibility. SIGUSR1 will no longer be registered in a future release, so please use SIGUSR2 to generate reports.
"""


COMPUTE_LOG = """
2018-01-12 21:00:02.939 49455 INFO ceilometer.agent.manager [-] Polling pollster network.outgoing.packets in the context of some_pollsters
2018-01-12 21:00:02.950 49455 INFO ceilometer.agent.manager [-] Polling pollster memory.usage in the context of some_pollsters
2018-01-12 21:00:02.953 49455 WARNING ceilometer.compute.pollsters.memory [-] Cannot inspect data of MemoryUsagePollster for xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx, non-fatal reason: Failed to inspect memory usage of instance <name=instance-name1, id=xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx>, can not get info from libvirt.
2018-01-12 21:00:02.957 49455 WARNING ceilometer.compute.pollsters.memory [-] Cannot inspect data of MemoryUsagePollster for yyyyyyyy-yyyy-yyyy-yyyy-yyyyyyyyyyyy, non-fatal reason: Failed to inspect memory usage of instance <name=instance-name2, id=yyyyyyyy-yyyy-yyyy-yyyy-yyyyyyyyyyyy>, can not get info from libvirt.
2018-01-12 21:00:02.963 49455 WARNING ceilometer.compute.pollsters.memory [-] Cannot inspect data of MemoryUsagePollster for zzzzzzzz-zzzz-zzzz-zzzz-zzzzzzzzzzzz, non-fatal reason: Failed to inspect memory usage of instance <name=instance-name3, id=zzzzzzzz-zzzz-zzzz-zzzz-zzzzzzzzzzzz>, can not get info from libvirt.
2018-01-12 21:00:02.970 49455 INFO ceilometer.agent.manager [-] Polling pollster disk.write.requests in the context of some_pollsters
2018-01-12 21:00:02.976 49455 INFO ceilometer.agent.manager [-] Polling pollster network.incoming.packets in the context of some_pollsters
2018-01-12 21:00:02.981 49455 INFO ceilometer.agent.manager [-] Polling pollster cpu in the context of some_pollsters
2018-01-12 21:00:03.014 49455 INFO ceilometer.agent.manager [-] Polling pollster network.incoming.bytes in the context of some_pollsters
2018-01-12 21:00:03.020 49455 INFO ceilometer.agent.manager [-] Polling pollster disk.read.requests in the context of some_pollsters
2018-01-12 21:00:03.041 49455 INFO ceilometer.agent.manager [-] Polling pollster network.outgoing.bytes in the context of some_pollsters
2018-01-12 21:00:03.062 49455 INFO ceilometer.agent.manager [-] Polling pollster disk.write.bytes in the context of some_pollsters
""".strip()


def test_ceilometer_central_log():
    log = CeilometerCentralLog(context_wrap(CENTRAL_LOG))
    assert len(log.get('INFO')) == 3
    assert sorted([i['raw_message'] for i in log.get_after(datetime(2016, 11, 9, 14, 38, 10))]) == sorted([
        "2016-11-09 14:38:10.040 31723 INFO ceilometer.pipeline [-] Config file: {'sources': [{'interval': 600, 'meters': ['*'], 'name': 'meter_source', 'sinks': ['meter_sink']}, {'interval': 600, 'meters': ['cpu'], 'name': 'cpu_source', 'sinks': ['cpu_sink', 'cpu_delta_sink']}, {'interval': 600, 'meters': ['disk.read.bytes', 'disk.read.requests', 'disk.write.bytes', 'disk.write.requests', 'disk.device.read.bytes', 'disk.device.read.requests', 'disk.device.write.bytes', 'disk.device.write.requests'], 'name': 'disk_source', 'sinks': ['disk_sink']}, {'interval': 600, 'meters': ['network.incoming.bytes', 'network.incoming.packets', 'network.outgoing.bytes', 'network.outgoing.packets'], 'name': 'network_source', 'sinks': ['network_sink']}], 'sinks': [{'publishers': ['notifier://'], 'transformers': None, 'name': 'meter_sink'}, {'publishers': ['notifier://'], 'transformers': [{'name': 'rate_of_change', 'parameters': {'target': {'scale': '100.0 / (10**9 * (resource_metadata.cpu_number or 1))', 'type': 'gauge', 'name': 'cpu_util', 'unit': '%'}}}], 'name': 'cpu_sink'}, {'publishers': ['notifier://'], 'transformers': [{'name': 'delta', 'parameters': {'target': {'name': 'cpu.delta'}, 'growth_only': True}}], 'name': 'cpu_delta_sink'}, {'publishers': ['notifier://'], 'transformers': [{'name': 'rate_of_change', 'parameters': {'source': {'map_from': {'name': '(disk\\.device|disk)\\.(read|write)\\.(bytes|requests)', 'unit': '(B|request)'}}, 'target': {'map_to': {'name': '\\1.\\2.\\3.rate', 'unit': '\\1/s'}, 'type': 'gauge'}}}], 'name': 'disk_sink'}, {'publishers': ['notifier://'], 'transformers': [{'name': 'rate_of_change', 'parameters': {'source': {'map_from': {'name': 'network\\.(incoming|outgoing)\\.(bytes|packets)', 'unit': '(B|packet)'}}, 'target': {'map_to': {'name': 'network.\\1.\\2.rate', 'unit': '\\1/s'}, 'type': 'gauge'}}}], 'name': 'network_sink'}]}",
        '2016-11-09 14:38:10.041 31723 INFO ceilometer.pipeline [-] detected decoupled pipeline config format'
    ])


def test_ceilometer_collector_log():
    log = CeilometerCollectorLog(context_wrap(COLLECTOR_LOG))
    assert len(log.get('INFO')) == 2
    assert list(log.get_after(datetime(2016, 11, 9, 14, 38, 0)))[0]['raw_message'] == '2016-11-09 14:38:07.280 31638 WARNING oslo_reports.guru_meditation_report [-] Guru meditation now registers SIGUSR1 and SIGUSR2 by default for backward compatibility. SIGUSR1 will no longer be registered in a future release, so please use SIGUSR2 to generate reports.'


def test_ceilometer_compute_log():
    log = CeilometerComputeLog(context_wrap(COMPUTE_LOG))
    assert len(log.get('WARNING')) == 3
    assert len(list(log.get_after(datetime(2018, 1, 12, 21, 0, 3)))) == 4
    assert [line for line in log.lines if 'Cannot inspect data of MemoryUsagePollster' in line][0] == '2018-01-12 21:00:02.953 49455 WARNING ceilometer.compute.pollsters.memory [-] Cannot inspect data of MemoryUsagePollster for xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx, non-fatal reason: Failed to inspect memory usage of instance <name=instance-name1, id=xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx>, can not get info from libvirt.'



"""
MDAdm - command ``/usr/sbin/mdadm -E {device}``
===============================================
"""

from insights.core import CommandParser
from insights.core.plugins import parser
from insights.parsers import split_kv_pairs
from insights.parsers import SkipException

from insights.specs import Specs


@parser(Specs.mdadm_E)
class MDAdmMetadata(CommandParser, dict):
    """
    Parser for output of ``mdadm -E`` for each MD device in system.

    This stores the information from the output in the following properties:
    * ``device`` - the name of the device after /dev/ - e.g. loop0

    Sample output::

        /dev/loop0:
        Magic : a92b4efc
        Version : 1.0
        Feature Map : 0x0
        Array UUID : 98e098ef:c8662ce2:2ed2aa5f:7f0416a9
        Name : 0
        Creation Time : Mon Jun 29 02:16:52 2020
        Raid Level : raid1
        Raid Devices : 2

        Avail Dev Size : 16383968 sectors (7.81 GiB 8.39 GB)
        Array Size : 1048576 KiB (1024.00 MiB 1073.74 MB)
        Used Dev Size : 2097152 sectors (1024.00 MiB 1073.74 MB)
        Super Offset : 16383984 sectors
        Unused Space : before=0 sectors, after=14286824 sectors
        State : clean
        Device UUID : 5e249ed9:a9ee800a:c09c963f:363a18d2

        Update Time : Mon Jun 29 02:19:56 2020
        Bad Block Log : 512 entries available at offset -8 sectors
        Checksum : 395066e8 - correct
        Events : 60

        Device Role : Active device 0
        Array State : AA ('A' == active, '.' == missing, 'R' == replacing)

    Examples:
        >>> mdadm.device
        '/dev/loop0'
        >>> mdadm["Device UUID"]
        '5e249ed9:a9ee800a:c09c963f:363a18d2'
        >>> mdadm["Events"]
        60
    """
    def parse_content(self, content):
        mdadm_dev = "/mdadm_-E_.dev."
        if mdadm_dev in self.file_path:
            self.device = '/dev/' + self.file_path.split(mdadm_dev)[1].strip()
        else:
            raise SkipException('Cannot parse device name from path {p}'.format(p=self.file_path))

        for key, val in split_kv_pairs(content, split_on=':').items():
            if val.isdigit():
                val = int(val)

            self[key] = val



from insights.parsers import lsof
from insights.tests import context_wrap

LSOF = """
lsof: avoiding stat(/): -b was specified.
COMMAND     PID  TID           USER   FD      TYPE             DEVICE  SIZE/OFF       NODE NAME
systemd       1                root  cwd       DIR              253,1      4096        128 /
systemd       1                root  rtd       DIR              253,1      4096        128 /
systemd       1                root  txt       REG              253,1   1230920    1440410 /usr/lib/systemd/systemd
systemd       1                root  mem       REG              253,1     37152  135529970 /usr/lib64/libnss_sss.so.2
meventd     674  688           root  txt       REG              253,1     36344  135317954 /usr/sbin/dmeventd
dmeventd    674  688           root  mem       REG              253,1     20032  134413763 /usr/lib64/libuuid.so.1.3.0
dmeventd    674  688           root  mem       REG              253,1    248584  135108725 /usr/lib64/libblkid.so.1.1.0
bioset      611                root  txt   unknown                                         /proc/611/exe
""".strip()

LSOF_GOOD_V1 = """
COMMAND    PID  TID           USER   FD      TYPE             DEVICE  SIZE/OFF       NODE NAME
systemd-l  602                root   14u      CHR              13,64       0t0       6406 /dev/input/event0
systemd-l  602                root   15u      CHR              13,65       0t0       6407 /dev/input/event1
systemd-l  602                root   16u      CHR                4,6       0t0       4694 /dev/tty6
dbus-daem  603                dbus    0r      CHR                1,3       0t0       4674 /dev/null
dbus-daem  603  615           dbus    0r      CHR                1,3       0t0       4674 /dev/null
abrt-watc 8619                root    0r      CHR                1,3       0t0       4674 /dev/null
wpa_suppl  641                root    0r      CHR                1,3       0t0       4674 /dev/null
polkitd    642             polkitd    0u      CHR                1,3       0t0       4674 /dev/null
polkitd    642             polkitd    1u      CHR                1,3       0t0       4674 /dev/null
polkitd    642             polkitd    2u      CHR                1,3       0t0       4674 /dev/null
gmain      642  643        polkitd    0u      CHR                1,3       0t0       4674 /dev/null
gmain      642  643        polkitd    1u      CHR                1,3       0t0       4674 /dev/null
gmain      642  643        polkitd    2u      CHR                1,3       0t0       4674 /dev/null
gdbus      642  646        polkitd    0u      CHR                1,3       0t0       4674 /dev/null
gdbus      642  646        polkitd    1u      CHR                1,3       0t0       4674 /dev/null
gdbus      642  646        polkitd    2u      CHR                1,3       0t0       4674 /dev/null
JS         642  648        polkitd    1r      BLK               8,16       0t0      16884 /dev/sda
JS         642  648        polkitd    1u      CHR                1,3       0t0       4674 /dev/null
JS         642  648        polkitd    2u      CHR                1,3       0t0       4674 /dev/null
""".strip()

columns = ["COMMAND", "PID", "TID", "USER", "FD", "TYPE", "DEVICE", "SIZE/OFF", "NODE", "NAME"]


def test_lsof():
    ctx = context_wrap(LSOF)
    d = list(lsof.Lsof(ctx).parse(LSOF.splitlines()))

    assert set(columns) == set([k for f in d for k in f.keys()])
    assert d[0] == {
        "COMMAND": "systemd",
        "PID": "1",
        "TID": "",
        "USER": "root",
        "FD": "cwd",
        "TYPE": "DIR",
        "DEVICE": "253,1",
        "SIZE/OFF": "4096",
        "NODE": "128",
        "NAME": "/"
    }

    # Spot checks
    assert d[3]["TID"] == ""
    assert d[4]["TID"] == "688"
    assert d[5]["TYPE"] == "REG"
    assert d[7]["NAME"] == "/proc/611/exe"

    assert d[-1] == {
        "COMMAND": "bioset",
        "PID": "611",
        "TID": "",
        "USER": "root",
        "FD": "txt",
        "TYPE": "unknown",
        "DEVICE": "",
        "SIZE/OFF": "",
        "NODE": "",
        "NAME": "/proc/611/exe"
    }


def test_lsof_good():
    ctx = context_wrap(LSOF_GOOD_V1)
    d = list(lsof.Lsof(ctx).parse(LSOF_GOOD_V1.splitlines()))
    assert d[0] == {
        "COMMAND": "systemd-l",
        "PID": "602",
        "TID": "",
        "USER": "root",
        "FD": "14u",
        "TYPE": "CHR",
        "DEVICE": "13,64",
        "SIZE/OFF": "0t0",
        "NODE": "6406",
        "NAME": "/dev/input/event0"
    }

    # Spot checks
    assert d[3]["TID"] == ""
    assert d[4]["TID"] == "615"
    assert d[5]["COMMAND"] == "abrt-watc"
    assert d[5]["PID"] == "8619"
    assert d[5]["TYPE"] == "CHR"
    assert d[7]["NAME"] == "/dev/null"

    assert d[-1] == {
        "COMMAND": "JS",
        "PID": "642",
        "TID": "648",
        "USER": "polkitd",
        "FD": "2u",
        "TYPE": "CHR",
        "DEVICE": "1,3",
        "SIZE/OFF": "0t0",
        "NODE": "4674",
        "NAME": "/dev/null"
    }


def test_lsof_scan():
    ctx = context_wrap(LSOF_GOOD_V1)
    # Scannable provided `any` method
    lsof.Lsof.any('systemd_commands', lambda x: 'systemd' in x['COMMAND'])
    # Scannable provided `collect` method
    lsof.Lsof.collect('polkitd_user', lambda x: x['USER'] == 'polkitd')
    # Lsof provided `collect_keys` method
    lsof.Lsof.collect_keys('root_stdin', USER='root', FD='0r', SIZE_OFF='0t0')
    l = lsof.Lsof(ctx)
    assert l.systemd_commands
    assert len(l.polkitd_user) == 12

    assert hasattr(l, 'root_stdin')
    assert len(l.root_stdin) == 2

    assert l.root_stdin[0]['COMMAND'] == 'abrt-watc'
    assert l.root_stdin[0]['PID'] == '8619'
    assert l.root_stdin[0]['TID'] == ''
    assert l.root_stdin[0]['USER'] == 'root'
    assert l.root_stdin[0]['FD'] == '0r'
    assert l.root_stdin[0]['TYPE'] == 'CHR'
    assert l.root_stdin[0]['DEVICE'] == '1,3'
    assert l.root_stdin[0]['SIZE/OFF'] == '0t0'
    assert l.root_stdin[0]['NODE'] == '4674'
    assert l.root_stdin[0]['NAME'] == '/dev/null'

    assert l.root_stdin[1]['COMMAND'] == 'wpa_suppl'
    assert l.root_stdin[1]['PID'] == '641'
    assert l.root_stdin[1]['TID'] == ''
    assert l.root_stdin[1]['USER'] == 'root'
    assert l.root_stdin[1]['FD'] == '0r'
    assert l.root_stdin[1]['TYPE'] == 'CHR'
    assert l.root_stdin[1]['DEVICE'] == '1,3'
    assert l.root_stdin[1]['SIZE/OFF'] == '0t0'
    assert l.root_stdin[1]['NODE'] == '4674'
    assert l.root_stdin[1]['NAME'] == '/dev/null'



from __future__ import print_function
import datetime
import sys
import inspect
from collections import namedtuple

from pprint import pprint
from insights import dr, datasource, rule, condition, incident, parser
from insights.core.context import ExecutionContext
from insights.formats import Formatter, FormatterAdapter, render


def _find_context(broker):
    for k, v in broker.instances.items():
        if inspect.isclass(k) and issubclass(k, ExecutionContext):
            return v


class MarkdownFormat(Formatter):
    """
    This class prints a markdown formatted of rule hits. It should be used
    as a context manager and given an instance of an
    ``insights.core.dr.Broker``. ``dr.run`` should be called within the context
    using the same broker.

    Args:
        broker (Broker): the broker to watch and provide a summary about.
        missing (bool): shows rules with missing dependencies if True.
        tracebacks (bool): shows tracebacks if any exceptions were raised.
            Useful for debugging.
        dropped (bool): Shows any files that weren't collected if running
            against an archive. Included for a corner case and typically not
            used in general.
        stream (file-like): Output is written to stream. Defaults to sys.stdout.
    """
    response = namedtuple('response', 'label title')

    def __init__(self,
                 broker,
                 missing=False,
                 tracebacks=False,
                 dropped=False,
                 show_rules=None,
                 stream=sys.stdout):
        super(MarkdownFormat, self).__init__(broker, stream=stream)
        self.missing = missing
        self.tracebacks = tracebacks
        self.dropped = dropped
        self.show_rules = [] if show_rules is None else show_rules

        self.counts = {'skip': 0, 'pass': 0, 'rule': 0, 'info': 0, 'metadata': 0, 'metadata_key': 0, 'fingerprint': 0, 'exception': 0}
        self.responses = {
            'skip': self.response(label="SKIP", title="Missing Deps: "),
            'pass': self.response(label="PASS", title="Passed      : "),
            'rule': self.response(label="FAIL", title="Failed      : "),
            'info': self.response(label="INFO", title="Info        : "),
            'metadata': self.response(label="META", title="Metadata    : "),
            'metadata_key': self.response(label="META", title="Metadata Key: "),
            'fingerprint': self.response(label="FINGERPRINT", title="Fingerprint : "),
            'exception': self.response(label="EXCEPT", title="Exceptions  : ")
        }

    def print_header(self, header, level):
        hdr = '#' * level + ' ' + header
        print(hdr, file=self.stream)

    def preprocess(self):
        self.broker.add_observer(self.count_exceptions, rule)
        self.broker.add_observer(self.count_exceptions, condition)
        self.broker.add_observer(self.count_exceptions, incident)
        self.broker.add_observer(self.count_exceptions, parser)

    def count_exceptions(self, c, broker):
        """
        Count exceptions as processing proceeds
        """
        if c in broker.exceptions:
            self.counts['exception'] += len(broker.exceptions[c])
        return self

    def show_tracebacks(self):
        """ Show tracebacks """
        if self.broker.tracebacks:
            print(file=self.stream)
            self.print_header("Tracebacks", 2)
            print("```", file=self.stream)
            for t in self.broker.tracebacks.values():
                print(t, file=self.stream)
            print("```", file=self.stream)

    def show_dropped(self):
        """ Show dropped files """
        ctx = _find_context(self.broker)
        if ctx and ctx.all_files:
            ds = self.broker.get_by_type(datasource)
            vals = []
            for v in ds.values():
                if isinstance(v, list):
                    vals.extend(d.path for d in v)
                else:
                    vals.append(v.path)
            dropped = set(ctx.all_files) - set(vals)
            print(file=self.stream)
            self.print_header("Dropped Files", 2)
            print("```", file=self.stream)
            pprint(dropped, indent=4, stream=self.stream)
            print("```", file=self.stream)

    def show_description(self):
        """ Prints the formatted response for the matching return type """
        def print_missing(c, v):
            resp = self.responses[v["type"]]
            name = "[%s] %s" % (resp.label, dr.get_name(c))
            self.print_header(name, 3)
            print(file=self.stream)
            print('*Missing Dependencies*:', file=self.stream)
            req_all, req_any = v.missing
            if req_all:
                print(file=self.stream)
                print('* Requires:', file=self.stream)
                for m in req_all:
                    print('    * {}'.format(dr.get_name(m)), file=self.stream)
            if req_any:
                print(file=self.stream)
                for m in req_any:
                    print('* At Least One Of:', file=self.stream)
                    if type(m) == list:
                        for c in m:
                            print('    * {}'.format(dr.get_name(c)), file=self.stream)
                    else:
                        print('    * {}'.format(dr.get_name(m)), file=self.stream)
            print(file=self.stream)

        def printit(c, v):
            resp = self.responses[v["type"]]
            name = "[%s] %s" % (resp.label, dr.get_name(c))
            self.print_header(name, 3)
            print("```", file=self.stream)
            print(render(c, v), file=self.stream)
            print("```", file=self.stream)
            print(file=self.stream)

        for c in sorted(self.broker.get_by_type(rule), key=dr.get_name):
            v = self.broker[c]
            _type = v.get('type')
            if _type in self.responses:
                self.counts[_type] += 1
            if _type:
                if self.missing and _type == 'skip':
                    print_missing(c, v)
                elif ((self.show_rules and _type in self.show_rules) or
                        (not self.show_rules and _type != 'skip')):
                    printit(c, v)
        print(file=self.stream)

        self.print_header("Rule Execution Summary", 2)
        print("```", file=self.stream)
        for c in self.counts:
            print(' ' + self.responses[c].title + str(self.counts[c]), file=self.stream)
        print("```", file=self.stream)

    def postprocess(self):
        self.print_header("Insights Core Run ({})".format(datetime.datetime.now().strftime('%x %X')), 1)
        print(file=self.stream)
        self.print_header("Command Line", 2)
        print("`{}`".format(" ".join(sys.argv)))
        print(file=self.stream)
        if self.tracebacks:
            self.show_tracebacks()
        if self.dropped:
            self.show_dropped()

        print(file=self.stream)
        self.print_header("Rules Executed", 2)
        self.show_description()


class MarkdownFormatAdapter(FormatterAdapter):
    """ Displays results in a human readable format. """

    @staticmethod
    def configure(p):
        p.add_argument("-t", "--tracebacks", help="Show stack traces.", action="store_true")
        p.add_argument("-d", "--dropped", help="Show collected files that weren't processed.", action="store_true")
        p.add_argument("-m", "--missing", help="Show missing requirements.", action="store_true")
        p.add_argument("-S", "--show-rules", nargs="+",
                       choices=["fail", "info", "pass", "metadata", "fingerprint"],
                       metavar="TYPE",
                       help="Show results per rule type(s).")
        p.add_argument("-F", "--fail-only",
                       help="Show FAIL results only. Conflict with '-m', will be dropped when using them together. This option is deprecated by '-S fail'",
                       action="store_true")

    def __init__(self, args=None):
        self.missing = args.missing
        fail_only = args.fail_only
        if args.missing and fail_only:
            print('Options conflict: -m and -F, drops -F', file=sys.stderr)
            fail_only = None
        self.show_rules = []  # Empty by default, means show ALL types
        if not args.show_rules and fail_only:
            self.show_rules = ['rule']
        elif args.show_rules:
            self.show_rules = [opt.replace('fail', 'rule') for opt in args.show_rules]
        self.tracebacks = args.tracebacks
        self.dropped = args.dropped

    def preprocess(self, broker):
        self.formatter = MarkdownFormat(broker,
                self.missing, self.tracebacks, self.dropped, self.show_rules)
        self.formatter.preprocess()

    def postprocess(self, broker):
        self.formatter.postprocess()



"""
Collect all the interesting data for analysis - Core version
"""
from __future__ import absolute_import
import os
import six
import logging
from insights import collect

from .constants import InsightsConstants as constants
from .data_collector import DataCollector
from .utilities import systemd_notify_init_thread

APP_NAME = constants.app_name
logger = logging.getLogger(__name__)


class CoreCollector(DataCollector):
    def __init__(self, *args, **kwargs):
        super(CoreCollector, self).__init__(*args, **kwargs)

    def run_collection(self, conf, rm_conf, branch_info, blacklist_report):
        '''
        Initialize core collection here and generate the
        output directory with collected data.
        '''
        # initialize systemd-notify thread
        systemd_notify_init_thread()

        if rm_conf is None:
            rm_conf = {}

        # add tokens to limit regex handling
        #   core parses blacklist for files and commands as regex
        if 'files' in rm_conf:
            for idx, f in enumerate(rm_conf['files']):
                rm_conf['files'][idx] = '^' + f + '$'

        if 'commands' in rm_conf:
            for idx, c in enumerate(rm_conf['commands']):
                rm_conf['commands'][idx] = '^' + c + '$'

        logger.debug('Beginning to run collection...')

        # only load files, keywords, components into core
        core_blacklist = {
            'commands': rm_conf.get('commands', []),
            'files': rm_conf.get('files', []),
            'components': rm_conf.get('components', [])
        }

        collected_data_path = collect.collect(tmp_path=self.archive.tmp_dir, rm_conf=core_blacklist, client_timeout=self.config.cmd_timeout)
        # update the archive dir with the reported data location from Insights Core
        if not collected_data_path:
            raise RuntimeError('Error running collection: no output path defined.')
        self.archive.archive_dir = collected_data_path
        self.archive.archive_name = os.path.basename(collected_data_path)

        if not six.PY3:
            # collect.py returns a unicode string, and these must be bytestrings
            #   when we call the tar command in 2.6
            self.archive.archive_dir = self.archive.archive_dir.encode('utf-8')
            self.archive.archive_name = self.archive.archive_name.encode('utf-8')

        # set hostname_path for soscleaner
        if os.path.exists(os.path.join(self.archive.archive_dir, 'data', 'insights_commands', 'hostname_-f')):
            self.hostname_path = 'data/insights_commands/hostname_-f'
        else:
            # fall back to hostname if hostname -f not available
            self.hostname_path = 'data/insights_commands/hostname'

        logger.debug('Collection finished.')

        self.redact(rm_conf)

        # collect metadata
        logger.debug('Collecting metadata...')
        self._write_branch_info(branch_info)
        self._write_display_name()
        self._write_ansible_host()
        self._write_version_info()
        self._write_tags()
        self._write_blacklist_report(blacklist_report)
        self._write_egg_release()
        logger.debug('Metadata collection finished.')



import doctest
import pytest
from insights.parsers import tuned_conf
from insights.parsers.tuned_conf import TunedConfIni
from insights.tests import context_wrap
from insights.contrib.ConfigParser import NoSectionError, NoOptionError

TUNED_CONF = """
#
# tuned configuration main service
#

[main]
# Interval for monitoring and tuning. Default is 10s.
interval=10

#
# Disk monitoring section
#
[DiskMonitor]
# Enabled or disable the plugin. Default is True. Any other value
# disables it.
# enabled=False

#
# Disk tuning section
#
[DiskTuning]
# Enabled or disable the plugin. Default is True. Any other value
# disables it.
# enabled=False
#hdparm=False
alpm=False

#
# Net monitoring section
#
[NetMonitor]
# Enabled or disable the plugin. Default is True. Any other value
# disables it.
 enabled=

#
# Net tuning section
#
[NetTuning]
# Enabled or disable the plugin. Default is True. Any other value
# disables it.
enabled=False

#
# CPU monitoring section
#
[CPUMonitor]
# Enabled or disable the plugin. Default is True. Any other value
# disables it.
enabled=False

#
# CPU tuning section
#
[CPUTuning]
# Enabled or disable the plugin. Default is True. Any other value
# disables it.
enabled=False
""".strip()


TUNED_CONF_DOCS = """
#
# Net tuning section
#
[NetTuning]
# Enabled or disable the plugin. Default is True. Any other value
# disables it.
 enabled=False

#
# CPU monitoring section
#
[CPUMonitor]
# Enabled or disable the plugin. Default is True. Any other value
# disables it.
# enabled=False
""".strip()


def test_documentation():
    env = {'tuned_obj': tuned_conf.TunedConfIni(context_wrap(TUNED_CONF_DOCS))}
    failed_count, tests = doctest.testmod(tuned_conf, globs=env)
    assert failed_count == 0


def test_tuned_conf():
    result = tuned_conf.TunedConfIni(context_wrap(TUNED_CONF))
    assert sorted(result.sections()) == sorted(['CPUMonitor', 'CPUTuning', 'DiskMonitor', 'DiskTuning', 'NetMonitor', 'NetTuning', 'main'])

    with pytest.raises(NoSectionError) as exc:
        tuned_obj = TunedConfIni(context_wrap(TUNED_CONF))
        assert not tuned_obj.get('Xyz', 'Abc')
    assert "No section: 'Xyz'" in str(exc)

    with pytest.raises(NoOptionError) as exc:
        tuned_obj = TunedConfIni(context_wrap(TUNED_CONF))
        assert not tuned_obj.get('NetTuning', 'Abc')
    assert "No option 'Abc' in section: 'NetTuning'" in str(exc)

    with pytest.raises(ValueError) as exc:
        tuned_obj = TunedConfIni(context_wrap(TUNED_CONF))
        assert not tuned_obj.getboolean('main', 'interval')

    assert "Not a boolean: 10" in str(exc)
    assert tuned_obj.get('CPUMonitor', 'enabled') == "False"
    assert tuned_obj.getboolean('CPUMonitor', 'enabled') is False
    assert tuned_obj.get('NetMonitor', 'enabled') == ""



from insights.tests import context_wrap
from insights.combiners.nmcli_dev_show import AllNmcliDevShow
from insights.combiners import nmcli_dev_show
from insights.parsers.nmcli import NmcliDevShow
import doctest

NMCLI_SHOW1 = """
GENERAL.DEVICE:                         eth0
GENERAL.TYPE:                           ethernet
GENERAL.HWADDR:                         00:1A:4A:16:02:E0
GENERAL.MTU:                            1500
GENERAL.STATE:                          100 (connected)
GENERAL.CONNECTION:                     System eth0
GENERAL.CON-PATH:                       /org/freedesktop/NetworkManager/ActiveConnection/1
WIRED-PROPERTIES.CARRIER:               on
IP4.ADDRESS[1]:                         10.72.37.85/23
IP4.GATEWAY:                            10.72.37.254
IP4.ROUTE[1]:                           dst = 0.0.0.0/0, nh = 10.72.37.254, mt = 100
IP4.ROUTE[2]:                           dst = 10.72.36.0/23, nh = 0.0.0.0, mt = 100
IP4.DNS[1]:                             10.72.17.5
IP4.DOMAIN[1]:                          gsslab.pek2.redhat.com
IP6.ADDRESS[1]:                         2620:52:0:4824:21a:4aff:fe16:2e0/64
IP6.ADDRESS[2]:                         fe80::21a:4aff:fe16:2e0/64
IP6.GATEWAY:                            fe80:52:0:4824::1fe
IP6.ROUTE[1]:                           dst = ff00::/8, nh = ::, mt = 256, table=255
IP6.ROUTE[2]:                           dst = fe80::/64, nh = ::, mt = 256
IP6.ROUTE[3]:                           dst = ::/0, nh = fe80:52:0:4824::1fe, mt = 1024
IP6.ROUTE[4]:                           dst = 2620:52:0:4824::/64, nh = ::, mt = 256

GENERAL.DEVICE:                         lo
GENERAL.TYPE:                           loopback
GENERAL.HWADDR:                         00:00:00:00:00:00
GENERAL.MTU:                            65536
GENERAL.STATE:                          10 (unmanaged)
GENERAL.CONNECTION:                     --
GENERAL.CON-PATH:                       --
IP4.ADDRESS[1]:                         127.0.0.1/8
IP4.GATEWAY:                            --
IP6.ADDRESS[1]:                         ::1/128
IP6.GATEWAY:                            --
""".strip()

NMCLI_SHOW2 = """
GENERAL.DEVICE:                         eth0
GENERAL.TYPE:                           ethernet
GENERAL.HWADDR:                         00:1A:4A:16:02:E0
GENERAL.MTU:                            1500
GENERAL.STATE:                          100 (connected)
GENERAL.CONNECTION:                     System eth0
GENERAL.CON-PATH:                       /org/freedesktop/NetworkManager/ActiveConnection/1
WIRED-PROPERTIES.CARRIER:               on
IP4.ADDRESS[1]:                         10.72.37.85/23
IP4.GATEWAY:                            10.72.37.254
IP4.ROUTE[1]:                           dst = 0.0.0.0/0, nh = 10.72.37.254, mt = 100
IP4.ROUTE[2]:                           dst = 10.72.36.0/23, nh = 0.0.0.0, mt = 100
IP4.DNS[1]:                             10.72.17.5
IP4.DOMAIN[1]:                          gsslab.pek2.redhat.com
IP6.ADDRESS[1]:                         2620:52:0:4824:21a:4aff:fe16:2e0/64
IP6.ADDRESS[2]:                         fe80::21a:4aff:fe16:2e0/64
IP6.GATEWAY:                            fe80:52:0:4824::1fe
IP6.ROUTE[1]:                           dst = ff00::/8, nh = ::, mt = 256, table=255
IP6.ROUTE[2]:                           dst = fe80::/64, nh = ::, mt = 256
IP6.ROUTE[3]:                           dst = ::/0, nh = fe80:52:0:4824::1fe, mt = 1024
IP6.ROUTE[4]:                           dst = 2620:52:0:4824::/64, nh = ::, mt = 256
""".strip()

NMCLI_SHOW3 = """
GENERAL.DEVICE:                         lo
GENERAL.TYPE:                           loopback
GENERAL.HWADDR:                         00:00:00:00:00:00
GENERAL.MTU:                            65536
GENERAL.STATE:                          10 (unmanaged)
GENERAL.CONNECTION:                     --
GENERAL.CON-PATH:                       --
IP4.ADDRESS[1]:                         127.0.0.1/8
IP4.GATEWAY:                            --
IP6.ADDRESS[1]:                         ::1/128
IP6.GATEWAY:                            --
""".strip()


def test_allnmcli1():
    nmcli_obj = NmcliDevShow(context_wrap(NMCLI_SHOW1))
    allnmcli_obj = AllNmcliDevShow(nmcli_obj, None)
    con_dev = allnmcli_obj.connected_devices
    assert sorted(con_dev) == sorted(['eth0'])
    assert allnmcli_obj['eth0']['IP4_GATEWAY'] == "10.72.37.254"
    assert allnmcli_obj['eth0']['IP4_DNS1'] == "10.72.17.5"
    assert allnmcli_obj['eth0']['STATE'] == "connected"
    assert allnmcli_obj['eth0']['CON-PATH'] == "/org/freedesktop/NetworkManager/ActiveConnection/1"
    assert len(allnmcli_obj['lo']) == 10


def test_allnmcli2():
    nmcli_obj1 = NmcliDevShow(context_wrap(NMCLI_SHOW2))
    nmcli_obj2 = NmcliDevShow(context_wrap(NMCLI_SHOW3))
    allnmcli_obj = AllNmcliDevShow(None, [nmcli_obj1, nmcli_obj2])
    con_dev = allnmcli_obj.connected_devices
    assert sorted(con_dev) == sorted(['eth0'])
    assert allnmcli_obj['eth0']['IP4_GATEWAY'] == "10.72.37.254"
    assert allnmcli_obj['eth0']['IP4_DNS1'] == "10.72.17.5"
    assert allnmcli_obj['eth0']['STATE'] == "connected"
    assert allnmcli_obj['eth0']['CON-PATH'] == "/org/freedesktop/NetworkManager/ActiveConnection/1"
    assert len(allnmcli_obj['lo']) == 10


def test_doc_examples():
    env = {
            'allnmclidevshow': AllNmcliDevShow(NmcliDevShow(context_wrap(NMCLI_SHOW1)), None),
          }
    failed, total = doctest.testmod(nmcli_dev_show, globs=env)
    assert failed == 0



"""
Get SSL Certificate Info
========================

This module contains the following parsers:

SatelliteCustomCaChain - command ``awk 'BEGIN { pipe="openssl x509 -noout -subject -enddate"} /^-+BEGIN CERT/,/^-+END CERT/ { print | pipe } /^-+END CERT/ { close(pipe); printf("\\n")}' /etc/pki/katello/certs/katello-server-ca.crt``
========================================================================================================================================================================================================================================
RhsmKatelloDefaultCACert - command ``openssl x509 -in /etc/rhsm/ca/katello-default-ca.pem -noout -issuer``
==========================================================================================================
"""

from insights import parser, CommandParser
from datetime import datetime
from insights.parsers import ParseException, SkipException
from insights.specs import Specs
from insights.parsers.certificates_enddate import CertificatesEnddate


def parse_openssl_output(content):
    """
    It parses the output of "openssl -in <single_certificate_file> -xxx".
    Currently it only supports the attributes which the output is in
    key=value pairs. It saves the cert info into a dict. The value of notBefore and
    notAfter are saved to an instance of ExpirationDate, which contains the date
    in string and datetime format.

    Raises:
        ParseException: when the output isn't in key=value format or
                        the notAfter or notBefore isn't expected format.
    """
    date_format = '%b %d %H:%M:%S %Y'
    data = {}
    for line in content:
        if '=' not in line:
            raise ParseException('The line %s is not in key=value format' % line)
        key, value = [item.strip() for item in line.split('=', 1)]
        if key in ['notBefore', 'notAfter']:
            value_without_tz = value.rsplit(" ", 1)[0]
            try:
                date_time = datetime.strptime(value_without_tz, date_format)
            except Exception:
                raise ParseException('The %s is not in %s format.' % (key, date_format))
            value = CertificatesEnddate.ExpirationDate(value_without_tz, date_time)
        data[key] = value
    return data


class CertificateInfo(CommandParser, dict):
    """
    Base class to parse the output of "openssl -in <single_certificate_file> -xxx".
    Currently it only supports the attributes which the output is in
    key=value pairs.

    Sample Output::

        issuer= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=a.b.c.com
        notBefore=Dec  7 07:02:33 2020 GMT
        notAfter=Jan 18 07:02:33 2038 GMT
        subject= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=a.b.c.com

    Examples:
        >>> type(cert)
        <class 'insights.parsers.ssl_certificate.CertificateInfo'>
        >>> 'issuer' in cert
        True
        >>> cert['issuer']
        '/C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=a.b.c.com'
        >>> cert['notBefore'].str
        'Dec  7 07:02:33 2020'

    Raises:
        SkipException: when the command output is empty.
    """

    def __init__(self, context):
        super(CertificateInfo, self).__init__(
            context,
            extra_bad_lines=['error opening certificate', 'unable to load certificate'])

    def parse_content(self, content):
        """
        This uses the :py:func:`insights.parsers.ssl_certificate.parse_openssl_output` function.
        See its documentation for parsing details.
        """

        self.update(parse_openssl_output(content))
        if not self:
            raise SkipException("There is not any info in the cert.")


class CertificateChain(CommandParser, list):
    """
    Base class to parse the output of "openssl -in <certificate_chain_file> -xxx".
    Blank line is added to distinguish different certs in the chain.
    Currently it only supports the attributes which the output is in
    key=value pairs.

    Sample Output::

        issuer= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=test.a.com
        subject= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=test.b.com
        notBefore=Dec  7 07:02:33 2020 GMT
        notAfter=Jan 18 07:02:33 2038 GMT

        issuer= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=test.c.com
        subject= /C=US/ST=North Carolina/O=Katello/OU=SomeOrgUnit/CN=test.d.com
        notBefore=Nov 30 07:02:42 2020 GMT
        notAfter=Jan 18 07:02:43 2018 GMT

    Examples:
        >>> type(certs)
        <class 'insights.parsers.ssl_certificate.CertificateChain'>
        >>> len(certs)
        2
        >>> certs.earliest_expiry_date.str
        'Jan 18 07:02:43 2018'
    """

    def parse_content(self, content):
        """
        Parse the content of cert chain file. And it saves the certs
        in a list of dict.
        This uses the :py:func:`insights.parsers.ssl_certificate.parse_openssl_output` function.
        See its documentation for parsing details.

        Attributes:
            earliest_expiry_date(ExpirationDate):
                The earliest expiry datetime of the certs in the chain.
                None when there isn't "notAfter" for all the certs
                in the chain.

        Raises:
            SkipException: when the command output is empty.
        """

        self.earliest_expiry_date = None
        start_index = 0
        for index, line in enumerate(content):
            if not line.strip():
                # one cert ends
                if start_index != index:
                    self.append(parse_openssl_output(content[start_index:index]))
                start_index = index + 1
            if index == len(content) - 1:
                self.append(parse_openssl_output(content=content[start_index:index + 1]))
        if not self:
            raise SkipException("There is not any info in the ca cert chain.")
        for one_cert in self:
            expire_date = one_cert.get('notAfter')
            if expire_date and (self.earliest_expiry_date is None or expire_date.datetime < self.earliest_expiry_date.datetime):
                self.earliest_expiry_date = expire_date


@parser(Specs.satellite_custom_ca_chain)
class SatelliteCustomCaChain(CertificateChain):
    """
    .. note::
        Please refer to its super-class :class:`insights.parsers.ssl_certificate.CertificateChain` for more
        details.

    Sample Output::

        subject= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=test.a.com
        notAfter=Jan 18 07:02:33 2038 GMT

        subject= /C=US/ST=North Carolina/O=Katello/OU=SomeOrgUnit/CN=test.b.com
        notAfter=Jan 18 07:02:43 2028 GMT

    Examples:
        >>> type(satellite_ca_certs)
        <class 'insights.parsers.ssl_certificate.SatelliteCustomCaChain'>
        >>> len(satellite_ca_certs)
        2
        >>> satellite_ca_certs.earliest_expiry_date.str
        'Jan 18 07:02:43 2028'
    """
    pass


@parser(Specs.rhsm_katello_default_ca_cert)
class RhsmKatelloDefaultCACert(CertificateInfo):
    """
    .. note::
        Please refer to its super-class :class:`insights.parsers.ssl_certificate.CertificateInfo` for more
        details.

    Sample Output::

        issuer= /C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=a.b.c.com

    Examples:
        >>> type(rhsm_katello_default_ca)
        <class 'insights.parsers.ssl_certificate.RhsmKatelloDefaultCACert'>
        >>> rhsm_katello_default_ca['issuer']
        '/C=US/ST=North Carolina/L=Raleigh/O=Katello/OU=SomeOrgUnit/CN=a.b.c.com'
    """
    pass



import doctest
import pytest
from insights.parsers import ansible_tower_settings, SkipException
from insights.tests import context_wrap


ANSIBLE_TOWER_CONFIG_CUSTOM = '''
AWX_CLEANUP_PATHS = False
LOGGING['handlers']['tower_warnings']['level'] = 'DEBUG'
'''.strip()

ANSIBLE_TOWER_CONFIG_CUSTOM_INVALID1 = '''
'''.strip()

ANSIBLE_TOWER_CONFIG_CUSTOM_INVALID2 = '''
AWX_CLEANUP_PATHS
'''.strip()


def test_ansible_tower_settings():
    conf = ansible_tower_settings.AnsibleTowerSettings(context_wrap(ANSIBLE_TOWER_CONFIG_CUSTOM))
    assert conf['AWX_CLEANUP_PATHS'] == 'False'

    with pytest.raises(SkipException) as exc:
        ansible_tower_settings.AnsibleTowerSettings(context_wrap(ANSIBLE_TOWER_CONFIG_CUSTOM_INVALID1))
    assert 'No Valid Configuration' in str(exc)

    with pytest.raises(SkipException) as exc:
        ansible_tower_settings.AnsibleTowerSettings(context_wrap(ANSIBLE_TOWER_CONFIG_CUSTOM_INVALID2))
    assert 'No Valid Configuration' in str(exc)


def test_ansible_tower_settings_documentation():
    failed_count, tests = doctest.testmod(
        ansible_tower_settings,
        globs={'conf': ansible_tower_settings.AnsibleTowerSettings(context_wrap(ANSIBLE_TOWER_CONFIG_CUSTOM))}
    )
    assert failed_count == 0



"""
saphostctrl - Commands ``saphostctrl``
======================================

Parsers included in this module are:

SAPHostCtrlInstances - Command ``saphostctrl -function GetCIMObject -enuminstances SAPInstance``
------------------------------------------------------------------------------------------------
"""
from insights import parser, CommandParser
from insights.parsers import ParseException, SkipException
from insights.specs import Specs
from insights.core.filters import add_filter


SAP_INST_FILTERS = [
        '******',
        'CreationClassName',
        'SID',
        'SystemNumber',
        'InstanceName',
        'Hostname',
        'SapVersionInfo',
        'FullQualifiedHostname'
]
add_filter(Specs.saphostctl_getcimobject_sapinstance, SAP_INST_FILTERS)


@parser(Specs.saphostctl_getcimobject_sapinstance)
class SAPHostCtrlInstances(CommandParser):
    """
    This class provides processing for the output of the
    ``/usr/sap/hostctrl/exe/saphostctrl -function GetCIMObject -enuminstances SAPInstance``
    command on SAP systems.

    Sample output of the command::

        *********************************************************
         CreationClassName , String , SAPInstance
         SID , String , D89
         SystemNumber , String , 88
         InstanceName , String , HDB88
         Hostname , String , hdb88
         FullQualifiedHostname , String , hdb88.example.com
         IPAddress , String , 10.0.0.88
         SapVersionInfo , String , 749, patch 211, changelist 1754007
        *********************************************************
         CreationClassName , String , SAPInstance
         SID , String , D90
         SystemNumber , String , 90
         InstanceName , String , HDB90
         Hostname , String , hdb90
         FullQualifiedHostname , String , hdb90.example.com
         IPAddress , String , 10.0.0.90
         SapVersionInfo , String , 749, patch 211, changelist 1754007

    Examples:
        >>> type(sap_inst)
        <class 'insights.parsers.saphostctrl.SAPHostCtrlInstances'>
        >>> sap_inst.data[-1]['CreationClassName']
        'SAPInstance'
        >>> sap_inst.data[-1]['SID']
        'D90'
        >>> sap_inst.data[-1]['SapVersionInfo']  # Note: captured as one string
        '749, patch 211, changelist 1754007'
        >>> sap_inst.data[0]['InstanceType']  # Inferred code from InstanceName
        'HDB'

    Attributes:
        data (list): List of dicts where keys are the lead name of each line and
            values are the string value.
        instances (list): The list of instances found in the cluster output.
        sids (list): The list of SID found in the cluster output.
        types (list): The list of instance types found in the cluster output.
    Raises:
        SkipException: When input is empty.
        ParseException: When input cannot be parsed.
    """
    REQUIRED_DIRECTIVES = (
        'CreationClassName',
        'SID',
        'SystemNumber',
        'InstanceName',
        'Hostname',
        'SapVersionInfo',
        'FullQualifiedHostname'
    )

    def parse_content(self, content):
        if not content:
            raise SkipException("Empty content")
        if len(content) == 1:
            raise ParseException("Incorrect content: '{0}'".format(content))

        self.data = []
        self.instances = []
        _current_instance = {}
        _sids = set()
        _types = set()

        def _update_instance(inst):
            for _dir in self.REQUIRED_DIRECTIVES:
                if _dir not in inst:
                    raise ParseException('Missing: "{0}"'.format(_dir))

            if not inst['InstanceName'].endswith(inst['SystemNumber']):
                raise ParseException(
                    'InstanceName: "{0}" missing match with SystemNumber: "{0}"'.format(inst['InstanceName'], inst['SystemNumber']))
            # InstanceType = The chars in InstanceName before the SystemNumber
            # subtract len(sysnumber) characters from instance name
            inst['InstanceType'] = inst['InstanceName'][0:-len(inst['SystemNumber'])]

        _current_instance = {}
        for line in (l.strip() for l in content):
            if line.startswith('******'):
                # Skip separator lines but save and reset current instance
                if _current_instance:
                    _update_instance(_current_instance)
                    self.instances.append(_current_instance['InstanceName'])
                    self.data.append(_current_instance)
                    _types.add(_current_instance['InstanceType'])
                    _sids.add(_current_instance['SID'])
                _current_instance = {}
                continue
            fields = [i.strip() for i in line.split(',', 2)]
            if len(fields) < 3:
                raise ParseException("Incorrect line: '{0}'".format(line))
            # TODO: if we see something other than 'String' in the second
            # field, we could maybe change its type, say to an integer?
            _current_instance[fields[0]] = fields[2]
        # the last instance
        if _current_instance:
            _update_instance(_current_instance)
            self.instances.append(_current_instance['InstanceName'])
            self.data.append(_current_instance)
            _types.add(_current_instance['InstanceType'])
            _sids.add(_current_instance['SID'])
        self.sids = list(_sids)
        self.types = list(_types)

    def __len__(self):
        return len(self.data)



"""
AnsibleTowerSettings - file ``/etc/tower/conf.d/*.py`` and ``/etc/tower/settings.py``
=====================================================================================
The AnsibleTowerSettings class parses the file ``/etc/tower/conf.d/*.py`` and
``/etc/tower/settings.py``.
"""
from insights import parser, get_active_lines, Parser
from insights.specs import Specs
from insights.parsers import SkipException


@parser(Specs.ansible_tower_settings)
class AnsibleTowerSettings(Parser, dict):
    """
    Class for content of ansible tower config file ``/etc/tower/conf.d/*.py`` and ``/etc/tower/settings.py``.

    Sample ``/etc/tower/conf.d/*.py`` file::

        AWX_CLEANUP_PATHS = False

    Raises:
        SkipException: the file is empty or there is no valid content

    Examples::
    >>> type(conf)
    <class 'insights.parsers.ansible_tower_settings.AnsibleTowerSettings'>
    >>> conf['AWX_CLEANUP_PATHS']
    'False'
    """

    def parse_content(self, content):
        """Parse content of of ansible tower config file ``/etc/tower/conf.d/*.py`` and ``/etc/tower/settings.py``"""
        if not content:
            raise SkipException("No Valid Configuration")
        data = {}
        for line in get_active_lines(content):
            if "=" in line:
                key, value = line.split("=")
                data[key.strip()] = value.strip()
        if not data:
            raise SkipException("No Valid Configuration")
        self.update(data)



from insights import rule, make_fail
from insights.parsers.hostname import Hostname

ERROR_KEY = "INSIGHTS_HEARTBEAT"
HEARTBEAT_UUID = "9cd6f607-6b28-44ef-8481-62b0e7773614"
HOST = "insights-heartbeat-" + HEARTBEAT_UUID


@rule(Hostname)
def is_insights_heartbeat(hostname):
    hostname = hostname.hostname
    if hostname == HOST:
        return make_fail(ERROR_KEY)



"""
PmLogSummary - Command ``pmlogsummary``
=======================================
"""

from insights import parser, CommandParser
from insights.parsers import SkipComponent
from insights.specs import Specs


def parse(data):
    """
    Parse a set of key/value pairs into a heirarchical dictionary of
    typed values

    Arguments:
        data (dict): Input dictionary of key/value pairs

    Returns:
        dict: Heirarchical dictionary with keys separated at "." and type
        conversion of the numerical values
    """
    result = {}

    def typed(x):
        try:
            return float(x)
        except Exception:
            return x

    def insert(k, v):
        cur = result
        key_parts = k.split(".")

        # walk down the structure to the correct leaf
        for part in key_parts:
            if part not in cur:
                cur[part] = {}
            cur = cur[part]

        # break the value apart and store it
        l, r = v.split(None, 1)
        cur["val"] = typed(l)
        cur["units"] = r.strip()

    def kvs():
        # deal with whitespace and high level splitting
        for line in data:
            line = line.strip()
            if line:
                yield line.split(None, 1)

    for k, v in kvs():
        insert(k, v)

    return result


@parser(Specs.pmlog_summary)
class PmLogSummary(CommandParser, dict):
    """
    Parser to parse the output of the ``pmlogsummary`` command

    Sample output of the command is::

        mem.util.used  3133919.812 Kbyte
        mem.physmem  3997600.000 Kbyte
        kernel.all.cpu.user  0.003 none
        kernel.all.cpu.sys  0.004 none
        kernel.all.cpu.nice  0.000 none
        kernel.all.cpu.steal  0.000 none
        kernel.all.cpu.idle  3.986 none
        disk.all.total  0.252 count / sec

    Output is parsed and stored as a dictionary.  Each value is
    stored as a dict in the form ``{'val': number or string, 'units': string}``.
    Keys are a hierarchy of the input key value split on the "." character.
    For instance input line "mem.util.used  3133919.812 Kbyte" is parsed
    as::

        {
            'mem': {
                'util': {
                    'used': {
                        'val': 3133919.812,
                        'units': 'Kbyte'
                    }
                }
            }
        }

    Example:
        >>> type(pmlog_summary)
        <class 'insights.parsers.pmlog_summary.PmLogSummary'>
        >>> 'mem' in pmlog_summary
        True
        >>> pmlog_summary['disk']['all']['total'] == {'val': 0.252, 'units': 'count / sec'}
        True
    """

    def parse_content(self, content):
        data = parse(content)

        if len(data) == 0:
            raise SkipComponent()

        self.update(data)



"""
checkin.conf - Files ``/etc/splice/checkin.conf``
=================================================

Parser for checkin.conf configuration file.

"""

from insights.specs import Specs

from .. import IniConfigFile, parser


@parser(Specs.checkin_conf)
class CheckinConf(IniConfigFile):
    """
    Class for parsing content of "/etc/splice/checkin.conf".

    Sample input::

        [logging]
        config = /etc/splice/logging/basic.cfg

        # this is used only for single-spacewalk deployments
        [spacewalk]
        # Spacewalk/Satellite server to use for syncing data.
        host=
        # Path to SSH private key used to connect to spacewalk host.
        ssh_key_path=
        login=swreport

        # these are used for multi-spacewalk deployments
        # [spacewalk_one]
        # type = ssh
        # # Spacewalk/Satellite server to use for syncing data.
        # host=
        # # Path to SSH private key used to connect to spacewalk host.
        # ssh_key_path=
        # login=swreport
        #
        # [spacewalk_two]
        # type = file
        # # Path to directory containing report output
        # path = /path/to/output

        [katello]
        hostname=localhost
        port=443
        proto=https
        api_url=/sam
        admin_user=admin
        admin_pass=admin
        #autoentitle_systems = False
        #flatten_orgs = False

    Examples:
        >>> list(checkin_conf.sections())
        [u'logging', u'spacewalk', u'katello']
        >>> checkin_conf.get('spacewalk', 'host')
        u''
    """
    pass



import pytest
import doctest
from datetime import datetime
from insights.parsers import certificates_enddate
from insights.tests import context_wrap
from insights.parsers import SkipException


CRT1 = """
/usr/bin/find: '/etc/origin/node': No such file or directory
/usr/bin/find: '/etc/origin/master': No such file or directory
notAfter=May 25 16:39:40 2019 GMT
FileName= /etc/origin/node/cert.pem
unable to load certificate
139881193203616:error:0906D066:PEM routines:PEM_read_bio:bad end line:pem_lib.c:802:
unable to load certificate
140695459370912:error:0906D06C:PEM routines:PEM_read_bio:no start line:pem_lib.c:703:Expecting: TRUSTED CERTIFICATE
notAfter=May 25 16:39:40 2019 GMT
FileName= /etc/pki/ca-trust/extracted/pem/email-ca-bundle.pem
notAfter=Dec  9 10:55:38 2017 GMT
FileName= /etc/pki/consumer/cert.pem
notAfter=Jan  1 04:59:59 2022 GMT
FileName= /etc/pki/entitlement/3343502840335059594.pem
notAfter=Aug 31 02:19:59 2017 GMT
FileName= /etc/pki/consumer/cert.pem
notAfter=Jan  1 04:59:59 2022 GMT
FileName= /etc/pki/entitlement/2387590574974617178.pem
""".strip()

CRT3 = """
FileName= /etc/origin/node/cert.pem
notAfter=May 25 16:39:40 2019 GMT
FileName= /etc/pki/ca-trust/extracted/pem/email-ca-bundle.pem
notAfter=Dec  9 10:55:38 2017 GMT
FileName= /etc/pki/consumer/cert.pem
""".strip()

CRT4 = """
notAfter=May 25 16:39:40 2019 GMT
FileName= /etc/pki/ca-trust/extracted/pem/email-ca-bundle.pem
unable to load certificate
140463633168248:error:0906D06C:PEM routines:PEM_read_bio:no start line:pem_lib.c:701:Expecting: TRUSTED CERTIFICATE
notAfter=Dec  9 10:55:38 2017 GMT
FileName= /etc/pki/consumer/cert.pem
notAfter=Jan  1 04:59:59 2022 GMT
""".strip()

CRT5 = """
notAfter=May 25 16:39:40 2019 GMT
FileName= /etc/pki/ca-trust/extracted/pem/email-ca-bundle.pem
notAfter=Dec  9 10:55:38 2017 GMT
unable to load certificate
140463633168248:error:0906D06C:PEM routines:PEM_read_bio:no start line:pem_lib.c:701:Expecting: TRUSTED CERTIFICATE
FileName= /etc/pki/consumer/cert.pem
notAfter=Jan  1 04:59:59 2022 GMT
""".strip()

CRT6 = """
notAfter=May 25 16:39:40 2019
FileName= /etc/pki/ca-trust/extracted/pem/email-ca-bundle.pem
notAfter=Dec  9 10:55:38 20 GMT
FileName= /etc/pki/consumer/cert.pem
""".strip()

PATH1 = "/etc/origin/node/cert.pem"

CRT7 = """
/usr/bin/find: '/etc/origin/node': No such file or directory
/usr/bin/find: '/etc/origin/master': No such file or directory
/usr/bin/find: '/etc/pki': No such file or directory
/usr/bin/find: '/etc/ipa': No such file or directory
"""


def test_certificates_enddate():

    Cert1 = certificates_enddate.CertificatesEnddate(context_wrap(CRT1))
    assert PATH1 in Cert1.certificates_path
    expiration_date = Cert1.expiration_date(PATH1)
    assert expiration_date.str == 'May 25 16:39:40 2019'
    assert expiration_date.datetime == datetime(2019, 5, 25, 16, 39, 40)

    Cert3 = certificates_enddate.CertificatesEnddate(context_wrap(CRT3))
    assert (set(Cert3.certificates_path) == set([
            '/etc/pki/consumer/cert.pem',
            '/etc/pki/ca-trust/extracted/pem/email-ca-bundle.pem']))

    Cert4 = certificates_enddate.CertificatesEnddate(context_wrap(CRT4))
    assert (set(Cert4.certificates_path) == set([
            '/etc/pki/consumer/cert.pem',
            '/etc/pki/ca-trust/extracted/pem/email-ca-bundle.pem']))

    Cert5 = certificates_enddate.CertificatesEnddate(context_wrap(CRT5))
    assert (set(Cert5.certificates_path) == set([
            '/etc/pki/ca-trust/extracted/pem/email-ca-bundle.pem']))


def test_certificates_enddate_unparsable_datatime():
    Cert6 = certificates_enddate.CertificatesEnddate(context_wrap(CRT6))
    assert (set(Cert6.certificates_path) == set([
            '/etc/pki/consumer/cert.pem',
            '/etc/pki/ca-trust/extracted/pem/email-ca-bundle.pem']))
    assert Cert6.expiration_date('/etc/pki/consumer/cert.pem').datetime is None
    assert (Cert6.expiration_date(
            '/etc/pki/ca-trust/extracted/pem/email-ca-bundle.pem').str ==
            'May 25 16:39:40')
    assert (Cert6.expiration_date(
        '/etc/pki/ca-trust/extracted/pem/email-ca-bundle.pem').datetime is None)
    assert (Cert6.expiration_date('/etc/pki/email-ca-bundle.pem') is None)


def test_doc():
    cert_enddate = certificates_enddate.CertificatesEnddate(context_wrap(CRT1))

    globs = {
        'cert_enddate': cert_enddate
    }
    failed, tested = doctest.testmod(certificates_enddate, globs=globs)
    assert failed == 0


def test_exception():
    with pytest.raises(SkipException):
        certificates_enddate.CertificatesEnddate(context_wrap(CRT7))



from insights.parsers.nsswitch_conf import NSSwitchConf
from insights.tests import context_wrap

NSSWITCH = """
# To use db, put the "db" in front of "files" for entries you want to be
# looked up first in the databases
#
# Example:
#passwd:    db files nisplus nis
#shadow:    db files nisplus nis
#group:     db files nisplus nis

passwd:     files sss
shadow:     files sss
group:      files sss
#initgroups: files

#hosts:     db files nisplus nis dns
hosts:      files dns myhostname

# Example - obey only what nisplus tells us...
#services:   nisplus [NOTFOUND=return] files
#networks:   nisplus [NOTFOUND=return] files
#protocols:  nisplus [NOTFOUND=return] files
#rpc:        nisplus [NOTFOUND=return] files
#ethers:     nisplus [NOTFOUND=return] files
#netmasks:   nisplus [NOTFOUND=return] files

bootparams: nisplus [NOTFOUND=return] files

"""

NSSWITCH_ERROR = """
passwd      files sss DNS
GROUP:      Files SSS
"""


def test_nsswitch_conf():
    nss = NSSwitchConf(context_wrap(NSSWITCH, path='/etc/nsswitch.conf'))
    assert hasattr(nss, 'data')
    assert hasattr(nss, 'errors')
    assert hasattr(nss, 'sources')

    assert sorted(nss.data.keys()) == sorted([
        'passwd', 'shadow', 'group', 'hosts', 'bootparams'
    ])
    assert nss.errors == []
    assert nss.sources == set([
        'files', 'sss', 'dns', 'myhostname', 'nisplus', '[notfound=return]'
    ])

    assert 'passwd' in nss
    assert 'initgroups' not in nss
    assert nss['passwd'] == 'files sss'
    assert nss['bootparams'] == 'nisplus [notfound=return] files'


def test_nsswitch_errors():
    nss = NSSwitchConf(context_wrap(NSSWITCH_ERROR, path='/etc/nsswitch.conf'))
    assert hasattr(nss, 'data')
    assert hasattr(nss, 'errors')
    assert hasattr(nss, 'sources')

    # Test that valid contents are stored in lower case
    assert nss.data == {'group': 'files sss'}
    # Invalid lines are stored as is
    assert nss.errors == [
        'passwd      files sss DNS'
    ]
    assert nss.sources == set(['files', 'sss'])



from insights.parsers import ls_ipa_idoverride_memberof
from insights.parsers.ls_ipa_idoverride_memberof import LsIPAIdoverrideMemberof
from insights.tests import context_wrap
import doctest

LS_IPA_IDOVERRIDEMEMBEROF = """
/usr/share/ipa/ui/js/plugins/idoverride-memberof:
total 0
drwxr-xr-x. 2 0 0 0 Nov 11 11:44 .
drwxr-xr-x. 4 0 0 0 Nov 11 11:44 ..
-rw-rw-r--. 1 0 0 0 Nov 11 11:44 idoverride-memberof.js
-rw-rw-r--. 1 0 0 0 Nov 11 11:44 idoverride-admemberof.js
"""


def test_ls_ipa_idoverride_memberof():
    ls_ipa_idoverride_memberof = LsIPAIdoverrideMemberof(context_wrap(LS_IPA_IDOVERRIDEMEMBEROF))
    assert '/usr/share/ipa/ui/js/plugins/idoverride-memberof' in ls_ipa_idoverride_memberof
    assert ls_ipa_idoverride_memberof.files_of('/usr/share/ipa/ui/js/plugins/idoverride-memberof') == ['idoverride-memberof.js', 'idoverride-admemberof.js']


def test_ls_ipa_idoverride_memberof_doc():
    failed_count, tests = doctest.testmod(
        ls_ipa_idoverride_memberof,
        globs={'ls_ipa_idoverride_memberof': ls_ipa_idoverride_memberof.LsIPAIdoverrideMemberof(context_wrap(LS_IPA_IDOVERRIDEMEMBEROF))}
    )
    assert failed_count == 0



"""
NeutronConf - file ``/etc/neutron/neutron.conf``
================================================

This class provides parsing for the file ``/etc/neutron/neutron.conf``.

Sample input data is in the format::

    [DEFAULT]
    # debug = False
    debug = False
    # verbose = True
    verbose = False
    core_plugin =neutron.plugins.openvswitch.ovs_neutron_plugin.OVSNeutronPluginV2

    [quotas]
    default_quota = -1
    quota_network = 10
    [agent]
    report_interval = 60

    [keystone_authtoken]
    auth_host = ost-controller-lb-del.om-l.dsn.inet
    auth_port = 35357

See the ``IniConfigFile`` class for examples.
"""
from .. import IniConfigFile, parser, add_filter
from insights.specs import Specs

add_filter(Specs.neutron_conf, ["["])

ADDITIONAL_FILTERS = [
    "service_plugins",
    "allow_automatic_dhcp_failover",
    "rpc_workers",
    "api_workers",
    "ipam_driver",
    "agent_down_time",
    "agent_report_interval",
    "router_distributed"
]
add_filter(Specs.neutron_conf, ADDITIONAL_FILTERS)


@parser(Specs.neutron_conf)
class NeutronConf(IniConfigFile):
    """Class to parse file ``neutron.conf``."""
    pass



"""
JBoss version - File ``$JBOSS_HOME/version.txt``
================================================

This module provides plugins access to file ``$JBOSS_HOME/version.txt``

Typical content of file ``$JBOSS_HOME/version.txt`` is::

    Red Hat JBoss Enterprise Application Platform - Version 6.4.3.GA

This module parses the file content and stores data in the dict ``self.parsed``.
The version info can also be got via ``obj.major`` and ``obj.minor``, etc.

Examples:
    >>> jboss_version.file_path
    '/home/test/jboss/jboss-eap-6.4/version.txt'
    >>> jboss_version.raw
    'Red Hat JBoss Enterprise Application Platform - Version 6.4.3.GA'
    >>> jboss_version.major
    6
    >>> jboss_version.minor
    4
    >>> jboss_version.release
    3
    >>> jboss_version.version
    '6.4.3'
    >>> jboss_version.code_name
    'GA'
"""
from .. import Parser, parser
from ..specs import Specs


@parser(Specs.jboss_version)
class JbossVersion(Parser):
    """Parses the content of file ``$JBOSS_HOME/version.txt``."""

    def parse_content(self, content):
        self.raw = content[0]
        product, _, version_name = [v.strip() for v in content[0].partition("- Version")]
        version, code_name = version_name.strip().rsplit(".", 1)
        major, minor, release = version.split(".")
        self.parsed = {
            "product": product,
            "version": version,
            "code_name": code_name,
            "major": int(major),
            "minor": int(minor),
            "release": int(release)
        }

    @property
    def version(self):
        """string: the version of this running JBoss progress."""
        return self.parsed["version"]

    @property
    def major(self):
        """int: the major version of this running JBoss progress."""
        return self.parsed["major"]

    @property
    def minor(self):
        """int: the minor version of this running JBoss progress."""
        return self.parsed["minor"]

    @property
    def release(self):
        """int: release of this running JBoss progress."""
        return self.parsed["release"]

    @property
    def code_name(self):
        """string: code name of this running JBoss progress."""
        return self.parsed["code_name"]
